#!/usr/bin/env python

import re
import os
import subprocess as sp

# vbox-enable-discard.py
# 2020, ed <irc.rizon.net>, MIT-licensed
# remounts all vdi files with ssd trim support

try:
	os.chdir(r"C:\Program Files\Oracle\VirtualBox")
except:
	pass

# "c:\Program Files\Oracle\VirtualBox\VBoxManage.exe" list vms
# "builder" {e91b6e92-796a-47ea-8928-39b0f0207c4d}
# "r0cbox" {809928e8-43d6-44e5-8097-43a4f37db650}

vms = []
cmd = ["VBoxManage", "list", "vms"]
for ln in sp.Popen(cmd, stdout=sp.PIPE).communicate()[0].split(b"\n"):
    ln = ln.decode("utf-8", "replace").strip().split(" ")[-1]
    if ln.startswith("{") and ln.endswith("}"):
        vms.append(ln)

# print(vms)
# ['{e91b6e92-796a-47ea-8928-39b0f0207c4d}', '{809928e8-43d6-44e5-8097-43a4f37db650}']

# "c:\Program Files\Oracle\VirtualBox\VBoxManage.exe" showvminfo --machinereadable "{e91b6e92-796a-47ea-8928-39b0f0207c4d}"
# name="builder"
# VMState="poweroff"
# "SATA-0-0"="C:\Users\ed\VirtualBox VMs\builder\builder.vdi"

re_vdi = re.compile(r'^"(SATA)-([0-9]+)-([0-9]+)"="(.*\.vdi)"$')
re_generic = re.compile(r"^([^=]+)=(.*)$")

skipped = []
for vm in vms:
    name = None
    state = None
    disks = []
    cmd = ["VBoxManage", "showvminfo", "--machinereadable", vm]
    for ln in sp.Popen(cmd, stdout=sp.PIPE).communicate()[0].split(b"\n"):
        ln = ln.decode("utf-8", "replace").strip()
        m = re_vdi.match(ln)
        if m:
            disks.append(m.groups())
            continue

        m = re_generic.match(ln)
        if not m:
            continue

        k, v = [x.strip('"') for x in m.groups()]

        if k == "name":
            name = v

        if k == "VMState":
            state = v

    if name == '<inaccessible>':
        continue
	
    if not name or not state:
        raise Exception(f"bad name [{repr(name)}] or state [{repr(state)}] for [{vm}]")

    if not disks:
        continue

    print(f"\n{state}: {name}")
    for disk in disks:
        print(f"  {disk}")

    # poweroff: builder
    #   ('SATA', '0', '0', 'C:\\Users\\ed\\VirtualBox VMs\\builder\\builder.vdi')

    if state != "poweroff":
        skipped.append([vm, name, state])
    else:
        for ctl, port, dev, path in disks:
            # fmt: off
            cmd = [
                "VBoxManage",
                "storageattach", vm,
                "--storagectl", ctl,
                "--port", port,
                "--device", dev,
                "--type", "hdd",
                "--medium", path,
                "--nonrotational", "on",
                "--discard", "on",
            ]
            # fmt: on
            sp.check_call(cmd)

print()
for vm, name, state in skipped:
    print(f'skipped {state} {vm} "{name}"')
